/*
 *  ---license-start
 *  eu-digital-covid-certificates / dcc-wallet-app-android
 *  ---
 *  Copyright (C) 2022 T-Systems International GmbH and all other contributors
 *  ---
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ---license-end
 *
 *  Created by mykhailo.nester on 25/03/2022, 23:02
 */

package dgca.wallet.app.android.vc.ui

import android.webkit.URLUtil
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.fasterxml.jackson.databind.ObjectMapper
import com.google.gson.Gson
import com.nimbusds.jose.JWSObject
import com.nimbusds.jose.Payload
import com.nimbusds.jose.crypto.factories.DefaultJWSVerifierFactory
import com.nimbusds.jose.jwk.ECKey
import dagger.hilt.android.lifecycle.HiltViewModel
import dgca.wallet.app.android.vc.*
import dgca.wallet.app.android.vc.data.VcRepository
import dgca.wallet.app.android.vc.data.remote.model.IssuerType
import dgca.wallet.app.android.vc.data.remote.model.Jwk
import dgca.wallet.app.android.vc.model.DataItem
import dgca.wallet.app.android.vc.model.PayloadData
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import timber.log.Timber
import java.net.URI
import java.text.ParseException
import java.time.ZonedDateTime
import javax.inject.Inject
import kotlin.math.roundToLong

@HiltViewModel
class VcViewModel @Inject constructor(
    private val vcRepository: VcRepository
) : ViewModel() {

    private val _event = MutableLiveData<Event<ViewEvent>>()
    val event: LiveData<Event<ViewEvent>> = _event

    private var jwsObject: JWSObject? = null
    private var issuerHolder: IssuerHolder? = null
    private var payloadUnzipString = ""
    private var kid = ""
    private var contextFileJson = ""

    init {
        setupPathProviders()
    }

    fun setContextJson(json: String) {
        contextFileJson = json
    }

    fun validate(input: String) {
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                decodeJwsInput(input)
            }
        }
    }

    fun issuerApproved() {
        viewModelScope.launch {
            val holder = issuerHolder ?: return@launch

            withContext(Dispatchers.IO) {
                val url = holder.issuerUrl
                val result = when (holder.type) {
                    IssuerType.HTTP -> vcRepository.resolveIssuer(url)
                    IssuerType.DID -> vcRepository.resolveIssuerByDid(url).map { it.publicKeyJwk }
                    else -> emptyList()
                }
                vcRepository.saveJWKs(result, url)
                validateJws(result.filter { it.kid == kid })
            }
        }
    }

    fun saveItem() {
        viewModelScope.launch(Dispatchers.IO) {
            val result = vcRepository.saveVcItem(
                kid,
                contextFileJson,
                payloadUnzipString,
                ZonedDateTime.now()
            )
            _event.postValue(Event(ViewEvent.OnSaveEvent(result)))
        }
    }

    private suspend fun decodeJwsInput(input: String) {
        jwsObject = decodeJws(input)

        val jws = jwsObject
        if (jws == null) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.JWS_STRUCTURE_NOT_VALID, payloadUnzipString)))
            return
        }

        val payloadObject = getPayload(jws)
        if (payloadObject == null) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.PAYLOAD_NOT_PARSED, payloadUnzipString)))
            return
        }

        val issuer = payloadObject[ISSUER] as String
        val notBefore = parseNbf(payloadObject[TIME_NOT_BEFORE])
        val expires = (payloadObject[TIME_EXPIRES] as? Double)?.roundToLong()

        kid = jws.header.keyID

        if (kid.isEmpty()) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.KID_NOT_INCLUDED, payloadUnzipString)))
            return
        }

        if (issuer.isEmpty()) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.ISSUER_NOT_INCLUDED, payloadUnzipString)))
            return
        }

        if (isTimeValid(notBefore, expires).not()) return
        val issuerHolder = resolveIssuerUrl(issuer)
        if (issuerHolder == null) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.ISSUER_NOT_RECOGNIZED, payloadUnzipString)))
            return
        }

        val isIssuerKnown = vcRepository.isIssuerKnown(issuerHolder.issuerUrl)
        if (isIssuerKnown.not()) {
            _event.postValue(Event(ViewEvent.OnIssuerNotTrusted(URI(issuerHolder.issuerUrl).host)))
            return
        }

        val result = vcRepository.getIssuerJWKsByKid(kid)
        if (result.isEmpty()) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.NO_JWK_FOR_KID, payloadUnzipString)))
            return
        }

        validateJws(result)
    }

    private fun decodeJws(jws: String): JWSObject? =
        try {
            JWSObject.parse(jws)
        } catch (ex: ParseException) {
            Timber.e(ex, "JWS parsing exception")
            null
        }

    private fun getPayload(jws: JWSObject): Map<String, Any>? {
        val zip = jws.header.customParams[KEY_ZIP]

        return if (zip == DEFLATE) {
            try {
                val payloadUnzip = inflate(jws.payload.toBytes())
                payloadUnzipString = payloadUnzip.toString(Charsets.UTF_8)
                Payload(payloadUnzip).toJSONObject()
            } catch (ex: Exception) {
                Timber.e(ex, "Failed to unzip payload")
                return null
            }
        } else {
            payloadUnzipString = jws.payload.toBytes().toString(Charsets.UTF_8)
            jws.payload.toJSONObject()
        }
    }

    private fun isTimeValid(notBefore: Long, expires: Long?): Boolean {
        val now = System.currentTimeMillis() / 1000
        if (now < notBefore || notBefore == -1L) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.TIME_BEFORE_NBF, payloadUnzipString)))
            return false
        }

        if (expires != null && now > expires) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.VC_EXPIRED, payloadUnzipString)))
            return false
        }

        return true
    }

    private fun resolveIssuerUrl(issuer: String): IssuerHolder? {
        issuerHolder = when {
            URLUtil.isValidUrl(issuer) -> IssuerHolder(issuer.resolveHttpUrl(), IssuerType.HTTP)
            issuer.contains(DID) -> IssuerHolder(issuer.resolveDidUrl(), IssuerType.DID)
            else -> null
        }
        return issuerHolder
    }

    private fun validateJws(list: List<Jwk>) {
        var isSignatureValid = false
        list.forEach {
            if (verifyJws(it, jwsObject)) {
                isSignatureValid = true
            }
        }

        if (!isSignatureValid) {
            _event.postValue(Event(ViewEvent.OnError(ErrorType.INVALID_SIGNATURE, payloadUnzipString)))
            return
        } else {
            parsePayload()
        }
    }

    private fun parsePayload() {
        val payloadData = Gson().fromJson(contextFileJson, PayloadData::class.java)

        val headers = mutableListOf<DataItem>()
        payloadData.header.forEach { (path, payloadItem) ->
            payloadUnzipString.tryFetchObject(path, payloadItem.title, headers)
        }

        val items = mutableListOf<DataItem>()
        payloadData.body.forEach { (path, payloadItem) ->
            payloadUnzipString.tryFetchObject(path, payloadItem.title, items)
        }

        _event.postValue(Event(ViewEvent.OnVerified(headers, items, payloadUnzipString)))
    }

    private fun verifyJws(jwk: Jwk, jws: JWSObject?): Boolean {
        jws ?: return false

        return try {
            val publicKey = ECKey.parse(ObjectMapper().writeValueAsString(jwk)).toECPublicKey()
            val verifier = DefaultJWSVerifierFactory().createJWSVerifier(jws.header, publicKey)

            val isVerified = jws.verify(verifier)
            Timber.d("JWS signature isValid: $isVerified")
            isVerified

        } catch (ex: Exception) {
            Timber.e(ex, "JWS signing verification exception")
            false
        }
    }

    private fun parseNbf(any: Any?): Long {
        any ?: return -1

        return try {
            val nbf = any.toString()
            if (nbf.contains(".")) {
                nbf.toDouble().toLong()
            } else {
                nbf.toLong()
            }
        } catch (ex: Exception) {
            return -1
        }
    }

    companion object {
        private const val KEY_ZIP = "zip"
        private const val ISSUER = "iss"
        private const val TIME_NOT_BEFORE = "nbf"
        private const val TIME_EXPIRES = "exp"
        private const val DEFLATE = "DEF"
        private const val DID = "did:web:"
    }

    sealed class ViewEvent {
        data class OnIssuerNotTrusted(val issuerDomain: String) : ViewEvent()
        data class OnError(val type: ErrorType, val rawPayloadData: String) : ViewEvent()
        data class OnVerified(val headers: MutableList<DataItem>, val payloadItems: List<DataItem>, val json: String) :
            ViewEvent()

        data class OnSaveEvent(val isSaved: Boolean) : ViewEvent()
    }

    enum class ErrorType {
        JWS_STRUCTURE_NOT_VALID,
        PAYLOAD_NOT_PARSED,
        KID_NOT_INCLUDED,
        ISSUER_NOT_RECOGNIZED,
        ISSUER_NOT_INCLUDED,
        TIME_BEFORE_NBF,
        VC_EXPIRED,
        NO_JWK_FOR_KID,
        INVALID_SIGNATURE
    }

    data class IssuerHolder(
        val issuerUrl: String,
        val type: IssuerType
    )
}